require 'rubygems'
require 'net/http'
require 'json'
require 'monitor.rb'
require 'fastthread'

class PresenceBot
  include MonitorMixin
  
  attr_accessor :status

  ONLINE = 'Online'
  OFFLINE = 'Offline'
  
  def initialize(msn_id, sleep_period=1)
    @msn_id = msn_id
    @refresh_period = sleep_period
    @my_thread = nil
    @continue = false;
    self.status = PresenceBot::OFFLINE
    
    super() #initialize MonitorMixin, no params
  end

  def continue?
    synchronize do
      @continue
    end
  end
  
  alias running? continue?

  def stop_now
    synchronize do
      @continue = false;
    end
  end

  def start_now
    synchronize do
      @continue = true;
    end
    @my_thread = Thread.new do
      run
    end
  end
  
  def online_event
    puts "#{Time.now}: You are online"
  end
  
  def offline_event
    puts "#{Time.now}: You are offline"
  end
  
  def tick_event
    #puts "#{Time.now}: You are offline"
  end
  
  def update_status (status)
    previous_status = self.status
    
    self.status = status
    
    if (previous_status != status)
      if (status == PresenceBot::ONLINE)
        online_event
      else 
        offline_event
      end
    end
    
    tick_event
    
  end
  
  def run
    url = URI.parse("http://messenger.services.live.com/users/#{@msn_id.downcase}@apps.messenger.live.com/presence")
    
    request = Net::HTTP::Get.new(url.path)
    
    http = Net::HTTP.start(url.host, url.port) 
    
    while continue?
      begin
        response =  http.request(request)
        
        result = JSON::parse(response.body) if response
        
        update_status result["status"]
        
      rescue Exception => e
        puts e
      end
      
      sleep @refresh_period
    end
    
    http.finish if http.start?
  end
  
end
